
/* ------------------------ speedlim ---------------------------- */
static t_class *speedlim_class;

typedef struct _speedlim
{
    t_object x_obj;
    t_clock  *x_clock;
    float    x_delay;
    int      x_is_waiting;
    int      x_has_changed;
    float    x_curval;
    float    x_lastout;
} t_speedlim;

static void speedlim_stop(t_speedlim *x)
{
    x->x_is_waiting = 0;
    x->x_has_changed = 0;
    clock_unset(x->x_clock);
}

static void speedlim_tick(t_speedlim *x)
{
    if(x->x_is_waiting)
    {
        x->x_is_waiting = 0;
	if(x->x_has_changed && (x->x_lastout != x->x_curval))
	{
            x->x_lastout = x->x_curval;
	    outlet_float(x->x_obj.ob_outlet, x->x_lastout);
	    x->x_is_waiting = 1;
            clock_delay(x->x_clock, x->x_delay);
	}
        x->x_has_changed = 0;
    }
}

static void speedlim_float(t_speedlim *x, t_float val)
{
    if(x->x_is_waiting)
    {
	x->x_curval = val;
	x->x_has_changed = 1;
    }
    else
    {
	if(val != x->x_lastout)
	{
	    x->x_is_waiting = 1;
            x->x_has_changed = 0;
            x->x_curval = val;
            x->x_lastout = x->x_curval;
	    outlet_float(x->x_obj.ob_outlet, x->x_lastout);
            clock_delay(x->x_clock, x->x_delay);
	}
    }
}

static void speedlim_ft1(t_speedlim *x, t_float delay)
{
    if(delay < 0.0)
        delay = 0.0;
    x->x_delay = delay;
}

static void speedlim_free(t_speedlim *x)
{
    clock_free(x->x_clock);
}

static void *speedlim_new(t_float delay)
{
    t_speedlim *x = (t_speedlim *)pd_new(speedlim_class);

    if(delay < 0.0)
        delay = 0.0;
    x->x_delay = delay;
    x->x_curval = 0.0;
    x->x_lastout = 0.0;
    x->x_is_waiting = 0;
    x->x_has_changed = 0;
    x->x_clock = clock_new(x, (t_method)speedlim_tick);
    outlet_new(&x->x_obj, &s_float);
    inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_float, gensym("ft1"));
    return (x);
}

static void speedlim_setup(void)
{
    speedlim_class = class_new(gensym("speedlim"), (t_newmethod)speedlim_new,
    	(t_method)speedlim_free, sizeof(t_speedlim), 0, A_DEFFLOAT, 0);
    class_addmethod(speedlim_class, (t_method)speedlim_stop, gensym("stop"), 0);
    class_addfloat(speedlim_class, (t_method)speedlim_float);
    class_addmethod(speedlim_class, (t_method)speedlim_ft1, gensym("ft1"), A_FLOAT, 0);
    class_sethelpsymbol(speedlim_class, gensym("iemhelp/help-speedlim"));
}

